@@ -23,7 +23,7 @@
 #   This specific module is licensed under the Perl Artistic License.
 #
 #
-#   $Id: File.pm,v 1.14 2004/11/25 15:30:36 fredo Exp $
+#   $Id: File.pm,v 1.21 2005/10/21 21:42:56 fredo Exp $
 #
 #=======================================================================
 package PDF::API2::Basic::PDF::File;
@@ -178,6 +178,7 @@ BEGIN
         $t =~ s|::|/|og;
         require "$t.pm";
     }
+    $PDF::API2::Basic::PDF::File::readDebug=0;
 }
 
 
@@ -198,6 +199,8 @@ use PDF::API2::Basic::PDF::Page;
 use PDF::API2::Basic::PDF::Pages;
 use PDF::API2::Basic::PDF::Null;
 
+no warnings qw[ deprecated recursion uninitialized ];
+
     # $Revisioning: 0.40_11 on Mon Oct  6 18:39:40 2003 $
     ( $VERSION ) = '0.40_10';
 
@@ -291,7 +294,7 @@ sub open
         $self->{' fname'} = $fname;
     }
     $fh->read($buf, 255);
-    if ($buf !~ m/^\%pdf\-1\.(\d)\s*$cr/moi)
+    if ($buf !~ m/^\%pdf\-1\.(\d)\s*/moi)
     { die "$fname not a PDF file version 1.x"; }
     else
     { $self->{' Version'} = $1; }
@@ -299,9 +302,12 @@ sub open
     $fh->seek(0, 2);            # go to end of file
     $end = $fh->tell();
     $self->{' epos'} = $end;
-    if (!$fh->seek(($end > 1024 ? $end - 1024 : 0, 0)))
+#    if (!$fh->seek(($end > 1024 ? $end - 1024 : 0, 0)))
+#      { die "Seek failed when reading PDF file $fname"; }
+#    $fh->read($buf, 1024);
+    if (!$fh->seek(($end > 64 ? $end - 64 : 0, 0)))
       { die "Seek failed when reading PDF file $fname"; }
-    $fh->read($buf, 1024);
+    $fh->read($buf, 64);
     if ($buf !~ m/startxref$cr([0-9]+)$cr\%\%eof.*?$/oi)
     { die "Malformed PDF file $fname"; }
     $xpos = $1;
@@ -350,9 +356,11 @@ sub open_swallowed
     $fh->seek(0, 2);            # go to end of file
     $end = $fh->tell();
     $self->{' epos'} = $end;
-    $fh->seek($end - 1024, 0);
-    $fh->read($buf, 1024);
-    if ($buf !~ m/startxref$cr\s*([0-9]+)$cr\%\%eof.*?$/oi)
+#    $fh->seek($end - 1024, 0);
+#    $fh->read($buf, 1024);
+    $fh->seek($end - 64, 0);
+    $fh->read($buf, 64);
+    if ($buf !~ m/startxref$cr\s*([0-9]+)$cr\%\%eof.*?/oi)
     { die "Malformed PDF file $fname"; }
     $xpos = $1;
 
@@ -565,28 +573,41 @@ sub readval
     my ($fh) = $self->{' INFILE'};
     my ($res, $key, $value, $k);
 
-    $str = update($fh, $str);
+    #$str = update($fh, $str);
+    updateRef($fh, \$str);
 
     if ($str =~ m/^<</so)
     {
         $str = substr ($str, 2);
-        $str = update($fh, $str);
+        #$str = update($fh, $str);
+        updateRef($fh, \$str);
         $res = PDFDict();
 
         while ($str !~ m/^>>/o) {
-            if ($str =~ s|^/($reg_char+)||o) {
+            if ($str =~ s|^/($reg_char+)||o) 
+            {
                 $k = PDF::API2::Basic::PDF::Name::name_to_string ($1, $self);
                 ($value, $str) = $self->readval($str, %opts);
                 $res->{$k} = $value;
-            } elsif($str =~ s|^/$ws_char+||o) { 
+            } 
+            elsif($str =~ s|^/$ws_char+||o) 
+            { 
                 # fixes a broken key problem of acrobat. -- fredo
                 ($value, $str) = $self->readval($str, %opts);
                 $res->{null} = $value;
+            } 
+            elsif($str =~ s|^//|/|o) 
+            { 
+                # fixes again a broken key problem of illustrator/enfocus. -- fredo
+                ($value, $str) = $self->readval($str, %opts);
+                $res->{null} = $value;
             }
-            $str = update($fh, $str); # thanks gareth.jones@stud.man.ac.uk
+            #$str = update($fh, $str); # thanks gareth.jones@stud.man.ac.uk
+            updateRef($fh, \$str); # thanks gareth.jones@stud.man.ac.uk
         }
         $str =~ s/^>>//o;
-        $str = update($fh, $str);
+        #$str = update($fh, $str);
+        updateRef($fh, \$str);
     # streams can't be followed by a lone carriage-return.
     # fredo: yes they can !!! -- use the MacOS Luke.
         if (($str =~ s/^stream(?:(?:\015\012)|\012|\015)//o)
@@ -607,7 +628,8 @@ sub readval
                 $value .= substr($str, 0, $k);
                 $res->{' stream'} = $value;
                 $res->{' nofilt'} = 1;
-                $str = update($fh, $str, 1);  # tell update we are in-stream and only need an endstream 
+                #$str = update($fh, $str, 1);  # tell update we are in-stream and only need an endstream 
+                updateRef($fh, \$str, 1);  # tell update we are in-stream and only need an endstream 
                 #$str =~ s/^endstream//o; # we cannot regexpr here since we need the first endstream only
                 # so we do the following:
                 my $wh = index($str,'endstream');
@@ -619,7 +641,8 @@ sub readval
                 if (defined $res->{'Type'} && defined $types{$res->{'Type'}->val});
     # gdj: FIXME: if any of the ws chars were crs, then the whole
     # string might not have been read.
-    } elsif ($str =~ m/^([0-9]+)$ws_char+([0-9]+)$ws_char+R/so) # objind
+    } 
+    elsif ($str =~ m/^([0-9]+)$ws_char+([0-9]+)$ws_char+R/so) # objind
     {
         $k = $1;
         $value = $2;
@@ -635,7 +658,8 @@ sub readval
         $res->{' realised'} = 0;
     # gdj: FIXME: if any of the ws chars were crs, then the whole
     # string might not have been read.
-    } elsif ($str =~ m/^([0-9]+)$ws_char+([0-9]+)$ws_char+obj/so)  # object data
+    } 
+    elsif ($str =~ m/^([0-9]+)$ws_char+([0-9]+)$ws_char+obj/so)  # object data
     {
         my ($obj);
         $k = $1;
@@ -650,7 +674,8 @@ sub readval
             $self->add_obj($res, $k, $value);
             $res->{' realised'} = 1;
         }
-        $str = update($fh, $str);       # thanks to kundrat@kundrat.sk
+        #$str = update($fh, $str);       # thanks to kundrat@kundrat.sk
+        updateRef($fh, \$str);       # thanks to kundrat@kundrat.sk
         $str =~ s/^endobj//o;
     } elsif ($str =~ m|^/($reg_char+)|so)        # name
     {
@@ -700,38 +725,47 @@ sub readval
     }
 
         $res = PDF::API2::Basic::PDF::String->from_pdf($value);
-    } elsif ($str =~ m/^</o)                                          # hex-string
+    } 
+    elsif ($str =~ m/^</o)                                          # hex-string
     {
         $str =~ s/^<//o;
         $fh->read($str, 255, length($str)) while (0 > index( $str, '>' ));
         ($value, $str) = ($str =~ /^(.*?)>(.*?)$/so);
         $res = PDF::API2::Basic::PDF::String->from_pdf("<" . $value . ">");
-    } elsif ($str =~ m/^\[/o)                                      # array
+    } 
+    elsif ($str =~ m/^\[/o)                                      # array
     {
         $str =~ s/^\[//o;
-        $str = update($fh, $str);
+        #$str = update($fh, $str);
+        updateRef($fh, \$str);
         $res = PDFArray();
         while ($str !~ m/^\]/o)
         {
             ($value, $str) = $self->readval($str, %opts);
             $res->add_elements($value);
+            #$str = update($fh, $str);   # str might just be exhausted!
+            updateRef($fh, \$str);   # str might just be exhausted!
         }
         $str =~ s/^\]//o;
-    } elsif ($str =~ m/^(true|false)$irreg_char/o)                        # boolean
+    } 
+    elsif ($str =~ m/^(true|false)$irreg_char/o)                        # boolean
     {
         $value = $1;
         $str =~ s/^(?:true|false)//o;
         $res = PDF::API2::Basic::PDF::Bool->from_pdf($value);
-    } elsif ($str =~ m/^([+-.0-9]+)$irreg_char/o)                             # number
+    } 
+    elsif ($str =~ m/^([+-.0-9]+)$irreg_char/o)                             # number
     {
         $value = $1;
         $str =~ s/^([+-.0-9]+)//o;
         $res = PDF::API2::Basic::PDF::Number->from_pdf($value);
-    } elsif ($str =~ m/^null$irreg_char/o)
+    } 
+    elsif ($str =~ m/^null$irreg_char/o)
     {
         $str =~ s/^null//o;
         $res = PDF::API2::Basic::PDF::Null->new;
-    } else
+    } 
+    else
     {
         # finally give up
         die "Can't parse `$str' near " . ($fh->tell()) . " length " . length($str) . ".";
@@ -1075,24 +1109,27 @@ comments.
 sub update
 {
     my ($fh, $str, $instream) = @_;
-
+    print STDERR "fpos=".tell($fh)." strlen=".length($str)."\n" if($readDebug);
     if($instream) {
         # we are inside a (possible binary) stream
         # so we fetch data till we see an 'endstream'
         # -- fredo/2004-09-03
         while ($str !~ m/endstream/o && !$fh->eof)
         {
+            print STDERR "fpos=".tell($fh)." strlen=".length($str)."\n" if($readDebug);
             $fh->read($str, 314, length($str));
         }
     } else {
         $str =~ s/^$ws_char*//o;
         while ($str !~ m/$cr/o && !$fh->eof)
         {
+            print STDERR "fpos=".tell($fh)." strlen=".length($str)."\n" if($readDebug);
             $fh->read($str, 314, length($str));
             $str =~ s/^$ws_char*//so;
         }
         while ($str =~ m/^\%/o) # restructured by fredo/2003-03-23
         {
+            print STDERR "fpos=".tell($fh)." strlen=".length($str)."\n" if($readDebug);
             $fh->read($str, 314, length($str)) while ($str !~ m/$cr/o && !$fh->eof);
             $str =~ s/^\%[^\015\012]+$ws_char*//so; # fixed for reportlab -- fredo
         }
@@ -1101,6 +1138,41 @@ sub update
     return $str;
 }
 
+sub updateRef
+{
+    my ($fh, $str, $instream) = @_;
+    my $inlen=16; my $inoff=1;
+    print STDERR "fpos=".tell($fh)." strlen=".length($$str)."\n" if($readDebug);
+    if($instream) {
+        # we are inside a (possible binary) stream
+        # so we fetch data till we see an 'endstream'
+        # -- fredo/2004-09-03
+        while ($$str !~ m/endstream/o && !$fh->eof)
+        {
+            print STDERR "fpos=".tell($fh)." strlen=".length($$str)."\n" if($readDebug);
+            $fh->read($$str, 314, length($$str));
+        }
+    } else {
+        $$str =~ s/^$ws_char*//o;
+        while ($$str !~ m/$cr/o && !$fh->eof && length($$str) < 16000)
+        {
+            print STDERR "fpos=".tell($fh)." strlen=".length($$str)."\n" if($readDebug);
+            $fh->read($$str, $inlen, length($$str));
+            $$str =~ s/^$ws_char*//so;
+            $inlen+=$inoff;
+        }
+        while ($$str =~ m/^\%/o) # restructured by fredo/2003-03-23
+        {
+            print STDERR "fpos=".tell($fh)." strlen=".length($$str)."\n" if($readDebug);
+            $fh->read($$str, $inlen, length($$str)) while ($$str !~ m/$cr/o && !$fh->eof);
+            $$str =~ s/^\%[^\015\012]+$ws_char*//so; # fixed for reportlab -- fredo
+            $inlen+=$inoff;
+        }
+    }
+
+    return undef;
+}
+
 
 =head2 $objind = $p->test_obj($num, $gen)
 
@@ -1168,7 +1240,10 @@ sub readxrtr
 
         $fh->read($buf, 20 * $xnum - $xdiff + 15, $xdiff);
         while ($xnum-- > 0 && $buf =~ s/^0*([0-9]*)$ws_char+0*([0-9]+)$ws_char+([nf])$cr//o)
-        { $xlist->{$xmin++} = [$1, $2, $3]; }
+        { 
+            $xlist->{$xmin} = [$1, $2, $3] unless(exists $xlist->{$xmin});
+            $xmin++;
+        }
     }
 
     if ($buf !~ /^trailer$cr/oi)
