@@ -21,7 +21,7 @@
 #   This specific module is licensed under the Perl Artistic License.
 #
 #
-#   $Id: Pages.pm,v 1.5 2004/06/15 09:13:13 fredo Exp $
+#   $Id: Pages.pm,v 1.7 2005/06/17 18:37:07 fredo Exp $
 #
 #=======================================================================
 package PDF::API2::Basic::PDF::Pages;
@@ -29,7 +29,7 @@ package PDF::API2::Basic::PDF::Pages;
 use strict;
 use vars qw(@ISA);
 @ISA = qw(PDF::API2::Basic::PDF::Dict);
-# no warnings qw(uninitialized);
+no warnings qw[ deprecated recursion uninitialized ];
 
 use PDF::API2::Basic::PDF::Dict;
 use PDF::API2::Basic::PDF::Utils;
@@ -257,11 +257,9 @@ sub find_prop
         { return $self->{$prop}; }
     } elsif (defined $self->{'Parent'})
     { return $self->{'Parent'}->find_prop($prop); }
+    return undef;
 }
 
-#    defined $_[0]->{$_[1]} && $_[0]->{$_[1]}->realised or
-#        defined $_[0]->{'Parent'} && $_[0]->{'Parent'}->find_prop($_[1]); }
-
 
 =head2 $p->add_font($pdf, $font)
 
