@@ -27,7 +27,7 @@
 #   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 #   Boston, MA 02111-1307, USA.
 #
-#   $Id: Page.pm,v 1.12 2005/06/17 18:53:33 fredo Exp $
+#   $Id: Page.pm,v 1.15 2005/11/02 18:18:20 fredo Exp $
 #
 #=======================================================================
 package PDF::API2::Page;
@@ -52,7 +52,7 @@ BEGIN {
     use POSIX qw(floor ceil);
     use Math::Trig;
 
-    ( $VERSION ) = sprintf '%i.%03i', split(/\./,('$Revision: 1.12 $' =~ /Revision: (\S+)\s/)[0]); # $Date: 2005/06/17 18:53:33 $
+    ( $VERSION ) = sprintf '%i.%03i', split(/\./,('$Revision: 1.15 $' =~ /Revision: (\S+)\s/)[0]); # $Date: 2005/11/02 18:18:20 $
 
 }
 
@@ -186,6 +186,29 @@ sub cropbox {
     $self;
 }
 
+=item ($llx, $lly, $urx, $ury) = $page->get_cropbox
+
+Gets the cropbox based one best estimates or the default.
+
+=cut
+
+sub get_cropbox 
+{
+    my ($self) = @_;
+    my $media = [ 0, 0, 612, 792 ];
+    foreach my $mediatype (qw[CropBox BleedBox TrimBox ArtBox MediaBox])
+    {
+        my $mediaobj = undef;
+        if($mediaobj = $self->find_prop($mediatype)) 
+        {
+            $media = [ map{ $_->val } $mediaobj->elementsof ];
+            last;
+        }
+    }
+
+    return(@{$media});
+}
+
 =item $page->bleedbox $w, $h
 
 =item $page->bleedbox $llx, $lly, $urx, $ury
@@ -214,6 +237,29 @@ sub bleedbox {
     $self;
 }
 
+=item ($llx, $lly, $urx, $ury) = $page->get_bleedbox
+
+Gets the bleedbox based one best estimates or the default.
+
+=cut
+
+sub get_bleedbox 
+{
+    my ($self) = @_;
+    my $media = [ 0, 0, 612, 792 ];
+    foreach my $mediatype (qw[BleedBox TrimBox ArtBox MediaBox CropBox])
+    {
+        my $mediaobj = undef;
+        if($mediaobj = $self->find_prop($mediatype)) 
+        {
+            $media = [ map{ $_->val } $mediaobj->elementsof ];
+            last;
+        }
+    }
+
+    return(@{$media});
+}
+
 =item $page->trimbox $w, $h
 
 =item $page->trimbox $llx, $lly, $urx, $ury
@@ -240,6 +286,29 @@ sub trimbox {
     $self;
 }
 
+=item ($llx, $lly, $urx, $ury) = $page->get_trimbox
+
+Gets the trimbox based one best estimates or the default.
+
+=cut
+
+sub get_trimbox 
+{
+    my ($self) = @_;
+    my $media = [ 0, 0, 612, 792 ];
+    foreach my $mediatype (qw[TrimBox ArtBox MediaBox CropBox BleedBox])
+    {
+        my $mediaobj = undef;
+        if($mediaobj = $self->find_prop($mediatype)) 
+        {
+            $media = [ map{ $_->val } $mediaobj->elementsof ];
+            last;
+        }
+    }
+
+    return(@{$media});
+}
+
 =item $page->artbox $w, $h
 
 =item $page->artbox $llx, $lly, $urx, $ury
@@ -268,6 +337,29 @@ sub artbox {
     $self;
 }
 
+=item ($llx, $lly, $urx, $ury) = $page->get_artbox
+
+Gets the artbox based one best estimates or the default.
+
+=cut
+
+sub get_artbox 
+{
+    my ($self) = @_;
+    my $media = [ 0, 0, 612, 792 ];
+    foreach my $mediatype (qw[ArtBox TrimBox BleedBox CropBox MediaBox])
+    {
+        my $mediaobj = undef;
+        if($mediaobj = $self->find_prop($mediatype)) 
+        {
+            $media = [ map{ $_->val } $mediaobj->elementsof ];
+            last;
+        }
+    }
+
+    return(@{$media});
+}
+
 =item $page->rotate $deg
 
 Rotates the page by the given degrees, which must be a multiple of 90.
@@ -369,8 +461,15 @@ sub annotation {
     my ($self, $type, $key, $obj) = @_;
 
     $self->{'Annots'}||=PDFArray();
-    $self->update if(ref($self->{'Annots'})=~/Objind/);
     $self->{'Annots'}->realise if(ref($self->{'Annots'})=~/Objind/);
+    if($self->{'Annots'}->is_obj($self->{' apipdf'}))
+    {
+        $self->{'Annots'}->update();
+    }
+    else
+    {
+        $self->update();
+    }
 
     my $ant=PDF::API2::Annotation->new;
     $self->{'Annots'}->add_elements($ant);
@@ -462,6 +561,15 @@ alfred reibenschuh
 =head1 HISTORY
 
     $Log: Page.pm,v $
+    Revision 1.15  2005/11/02 18:18:20  fredo
+    added get_(crop/bleed/trim/art)box methods
+
+    Revision 1.14  2005/08/06 20:59:10  fredo
+    fixed anotation pdf-array again for other braindead softs
+
+    Revision 1.13  2005/06/17 19:43:47  fredo
+    fixed CPAN modulefile versioning (again)
+
     Revision 1.12  2005/06/17 18:53:33  fredo
     fixed CPAN modulefile versioning (dislikes cvs)
 
@@ -499,4 +607,4 @@ alfred reibenschuh
     added CVS id/log
 
 
-=cut
\ No newline at end of file
+=cut
