@@ -27,7 +27,7 @@
 #   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 #   Boston, MA 02111-1307, USA.
 #
-#   $Id: CIDFont.pm,v 1.14 2005/06/17 19:44:03 fredo Exp $
+#   $Id: CIDFont.pm,v 1.15 2005/10/20 21:05:05 fredo Exp $
 #
 #=======================================================================
 package PDF::API2::Resource::CIDFont;
@@ -48,7 +48,7 @@ BEGIN {
 
     @ISA = qw( PDF::API2::Resource::BaseFont );
 
-    ( $VERSION ) = sprintf '%i.%03i', split(/\./,('$Revision: 1.14 $' =~ /Revision: (\S+)\s/)[0]); # $Date: 2005/06/17 19:44:03 $
+    ( $VERSION ) = sprintf '%i.%03i', split(/\./,('$Revision: 1.15 $' =~ /Revision: (\S+)\s/)[0]); # $Date: 2005/10/20 21:05:05 $
 
 }
 no warnings qw[ deprecated recursion uninitialized ];
@@ -139,8 +139,17 @@ sub width {
 sub width_cid {
     my ($self,$text)=@_;
     my $width=0;
+    my $lastglyph=0;
     foreach my $n (unpack('n*',$text)) {
         $width+=$self->wxByCId($n);
+        if($self->{-dokern} && $self->haveKernPairs())
+        {
+            if($self->kernPairCid($lastglyph, $n))
+            {
+                $width-=$self->kernPairCid($lastglyph, $n);
+            }
+        }
+        $lastglyph=$n;                    
     }
     $width/=1000;
     return($width);
@@ -204,11 +213,22 @@ sub textByStr
     return($self->text_cid($self->cidsByStr($text)));
 }
 
+sub textByStrKern 
+{
+    my ($self,$text,$size)=@_;
+    return($self->text_cid_kern($self->cidsByStr($text),$size));
+}
+
 sub text 
 { 
     my ($self,$text,$size)=@_;
     my $newtext=$self->textByStr($text);
-    if(defined $size)
+    if(defined $size && $self->{-dokern})
+    {
+        $newtext=$self->textByStrKern($text,$size);
+        return($newtext);
+    }
+    elsif(defined $size)
     {
         return("$newtext Tj");
     }
@@ -238,7 +258,62 @@ sub text_cid
     }
 }
 
-sub encodeByName {
+sub text_cid_kern 
+{
+    my ($self,$text,$size)=@_;
+    if(UNIVERSAL::can($self,'fontfile'))
+    {
+        foreach my $g (unpack('n*',$text)) 
+        {
+            $self->fontfile->subsetByCId($g);
+        }
+    }
+    if(defined $size && $self->{-dokern} && $self->haveKernPairs())
+    {
+            my $newtext=' ';
+            my $lastglyph=0;
+            my $tBefore=0;
+            foreach my $n (unpack('n*',$text)) 
+            {
+                if($self->kernPairCid($lastglyph, $n))
+                {
+                    $newtext.='> ' if($tBefore);
+                    $newtext.=sprintf('%i ',$self->kernPairCid($lastglyph, $n));
+                    $tBefore=0;
+                }
+                $lastglyph=$n;
+                my $t=sprintf('%04X',$n);
+                $newtext.='<' if(!$tBefore);
+                $newtext.=$t;
+                $tBefore=1;
+            }
+            $newtext.='> ' if($tBefore);
+            return("[ $newtext ] TJ");
+    }
+    elsif(defined $size)
+    {
+        my $newtext=unpack('H*',$text);
+        return("<$newtext> Tj");
+    }
+    else
+    {
+        my $newtext=unpack('H*',$text);
+        return("<$newtext>");
+    }
+}
+
+sub kernPairCid 
+{
+    return(0);
+}
+
+sub haveKernPairs 
+{
+    return(0);
+}
+
+sub encodeByName 
+{
     my ($self,$enc) = @_;
     return if($self->issymbol);
 
@@ -254,9 +329,14 @@ sub encodeByName {
     return($self);
 }
 
+sub kernPairCid {
+    return(0);
+}
+
 sub subsetByCId {
     return(1);
 }
+
 sub subvec {
     return(1);
 }
@@ -289,6 +369,9 @@ alfred reibenschuh
 =head1 HISTORY
 
     $Log: CIDFont.pm,v $
+    Revision 1.15  2005/10/20 21:05:05  fredo
+    added handling of optional kerning
+
     Revision 1.14  2005/06/17 19:44:03  fredo
     fixed CPAN modulefile versioning (again)
 
