@@ -38,7 +38,7 @@
 #   FREE SOFTWARE FOUNDATION, INC., 59 TEMPLE PLACE - SUITE 330,
 #   BOSTON, MA 02111-1307, USA.
 #
-#   $Id: CoreFont.pm,v 1.16 2005/10/01 22:41:07 fredo Exp $
+#   $Id: CoreFont.pm,v 1.17 2005/10/19 19:15:12 fredo Exp $
 #
 #=======================================================================
 package PDF::API2::Resource::Font::CoreFont;
@@ -76,7 +76,7 @@ BEGIN {
 
     @ISA=qw(PDF::API2::Resource::Font);
 
-    ( $VERSION ) = sprintf '%i.%03i', split(/\./,('$Revision: 1.16 $' =~ /Revision: (\S+)\s/)[0]); # $Date: 2005/10/01 22:41:07 $
+    ( $VERSION ) = sprintf '%i.%03i', split(/\./,('$Revision: 1.17 $' =~ /Revision: (\S+)\s/)[0]); # $Date: 2005/10/19 19:15:12 $
 
 }
 no warnings qw[ deprecated recursion uninitialized ];
@@ -242,10 +242,10 @@ sub new
     $self = $class->SUPER::new($pdf, $data->{apiname}.pdfkey().'~'.time());
     $pdf->new_obj($self) unless($self->is_obj($pdf));
     $self->{' data'}=$data;
+    $self->{-dokern}=1 if($opts{-dokern});
 
     $self->{'Subtype'} = PDFName($self->data->{type});
     $self->{'BaseFont'} = PDFName($self->fontname);
-
     if($opts{-pdfname}) 
     {
         $self->name($opts{-pdfname});
@@ -464,6 +464,9 @@ alfred reibenschuh
 =head1 HISTORY
 
     $Log: CoreFont.pm,v $
+    Revision 1.17  2005/10/19 19:15:12  fredo
+    added handling of optional kerning
+
     Revision 1.16  2005/10/01 22:41:07  fredo
     fixed font-naming race condition for multiple document updates
 
